
/*
 *
 * (C) Copyright IBM Corp. 1998-2005 - All Rights Reserved
 *
 */

#include "LETypes.h"
#include "LayoutEngine.h"
#include "OpenTypeLayoutEngine.h"
#include "MalayalamLayoutEngine.h"
#include "ScriptAndLanguageTags.h"

#include "GlyphSubstitutionTables.h"
#include "GlyphDefinitionTables.h"
#include "GlyphPositioningTables.h"

#include "GDEFMarkFilter.h"
#include "LEGlyphStorage.h"

#include "IndicReordering.h"

#include <stdio.h>

U_NAMESPACE_BEGIN

UOBJECT_DEFINE_RTTI_IMPLEMENTATION(MalayalamOpenTypeLayoutEngine)

void LEGlyphStorage::appendGlyphStorage(LEGlyphStorage &from)
{
    if (fInsertionList) applyInsertions();
    if (from.fInsertionList) from.applyInsertions();
    if ((!fInsertionList) && (from.fInsertionList))
    {
        fInsertionList = from.fInsertionList;
        from.fInsertionList = NULL;
    }

    if (!from.fGlyphCount)
        return;

    le_int32 newGlyphCount = fGlyphCount + from.fGlyphCount;

    fGlyphs = (LEGlyphID*)LE_GROW_ARRAY(fGlyphs, newGlyphCount);
    LE_ARRAY_COPY(fGlyphs+fGlyphCount, from.fGlyphs, from.fGlyphCount);

    le_int32 nLargestIndex = 0;
    if (fGlyphCount)
    {
        for (le_int32 i = 0; i < fGlyphCount; ++i)
        {
            if (fCharIndices[i] > nLargestIndex)
                nLargestIndex = fCharIndices[i];
        }
        nLargestIndex+=1;
    }
    fCharIndices = (le_int32 *)LE_GROW_ARRAY(fCharIndices, newGlyphCount);
    for (le_int32 i = 0; i < from.fGlyphCount; ++i)
        fCharIndices[fGlyphCount+i] = from.fCharIndices[i] + nLargestIndex;

    fAuxData = (le_uint32 *)LE_GROW_ARRAY(fAuxData, newGlyphCount);
    LE_ARRAY_COPY(fAuxData+fGlyphCount, from.fAuxData, from.fGlyphCount);

    fGlyphCount = newGlyphCount;
}

le_int32 MalayalamOpenTypeLayoutEngine::glyphPostProcessing(LEGlyphStorage &tempGlyphStorage, LEGlyphStorage &glyphStorage, LEErrorCode &success)
{
    if (LE_FAILURE(success)) {
        return 0;
    }

    glyphStorage.appendGlyphStorage(tempGlyphStorage);

    return glyphStorage.getGlyphCount();
}


le_int32 MalayalamOpenTypeLayoutEngine::computeGlyphs(const LEUnicode chars[], le_int32 offset, le_int32 count, le_int32 max, le_bool rightToLeft, LEGlyphStorage &glyphStorage, LEErrorCode &success)
{
    if (LE_FAILURE(success)) {
        return 0;
    }

    if (chars == NULL || offset < 0 || count < 0 || max < 0 || offset >= max || offset + count > max) {
        success = LE_ILLEGAL_ARGUMENT_ERROR;
        return 0;
    }

    le_int32 outGlyphCount=0;

    const IndicClassTable *classTable = IndicClassTable::getScriptClassTable(fScriptCode);
    le_int32 prev = 0;
    while (prev < count)
    { 
        le_int32 outCharCount=0, fakeGlyphCount=0;
        LEUnicode *outChars = NULL;
        LEGlyphStorage fakeGlyphStorage;

        le_int32 syllable = IndicReordering::findSyllable(classTable, chars+offset, prev, count);
        outCharCount = characterProcessing(chars+prev, offset, syllable-prev, max, rightToLeft, outChars, fakeGlyphStorage, success);

        if (LE_FAILURE(success)) {
            return 0;
        }

        if (outChars != NULL) {
            fakeGlyphCount = glyphProcessing(outChars, 0, outCharCount, outCharCount, rightToLeft, fakeGlyphStorage, success);
            LE_DELETE_ARRAY(outChars); // FIXME: a subclass may have allocated this, in which case this delete might not work...
        } else {
            fakeGlyphCount = glyphProcessing(chars+prev, offset, syllable-prev, max, rightToLeft, fakeGlyphStorage, success);
        }

        if (LE_FAILURE(success)) {
            return 0;
        }

        outGlyphCount = glyphPostProcessing(fakeGlyphStorage, glyphStorage, success);

        prev = syllable;
    }

    return outGlyphCount;
}

U_NAMESPACE_END
